# __version__.py
# autogenerated by Keats==0.1.0
__version__ = "0.1.1"
__name__ = "keats"
__title__ = "keats"
__authors__ = ["jvrana <justin.vrana@gmail.com>"]
__repo__ = None
__homepage__ = None
__description__ = ""
